@@ -3,7 +3,10 @@ from sys import argv
 if __name__ == '__main__':
     import rwkvstic.preQuantize as pq
     import rwkvstic.preJax as pj
-    if (argv[-1] == "--pq"):
-        pq.preQuantized()
+    args = {
+        "chunksize": 32 if "--cs" not in argv else int(argv[-1].split("=")[1]),
+        "useLogFix": "--logfix" in argv,
+    }
+    pq.preQuantized(**args)
     if (argv[-1] == "--preJax"):
         pj.preJax()
