@@ -4,8 +4,8 @@ import rwkvstic.agnostic.backends.base as RWKVOp
 
 
 class RWKVNumpyOps(RWKVOp.module):
-    def __init__(self, layers, embed):
-        super().__init__(layers, embed)
+    def __init__(self, layers, embed, *args, **kwargs):
+        super().__init__(layers, embed, *args, **kwargs)
 
         self.initTensor = lambda x: x.float().cpu().numpy()
         self.sqrt = np.sqrt
@@ -16,6 +16,7 @@ class RWKVNumpyOps(RWKVOp.module):
         self.matvec = np.matmul
         self.lerp = lambda x, y, z: x*(1-z) + y*(z)
         self.minimum = lambda x, y: np.minimum(x, y)
+        self.minimum = lambda x, y: np.maximum(x, y)
         self.klimit = [18] * embed
         # module def
         self.module = object
@@ -33,13 +34,13 @@ class RWKVNumpyOps(RWKVOp.module):
 
             return w*(xee2/x2) + b
         self.layernorm = ln
-        self.emptyState = [[0.01]*embed]*4*layers
+        self.emptyState = [[0.01]*embed]*(4+self.useLogFix)*layers
 
 
 class RWKVJaxOps(RWKVOp.module):
-    def __init__(self, layers, embed, preJax=False):
+    def __init__(self, layers, embed, *args, preJax=False, **kwargs):
         from jax import numpy as npjax
-        super().__init__(layers, embed)
+        super().__init__(layers, embed, *args, **kwargs)
         if preJax:
             self.initTensor = lambda x: npjax.array(x)
         else:
@@ -52,6 +53,7 @@ class RWKVJaxOps(RWKVOp.module):
         self.matvec = npjax.matmul
         self.lerp = lambda x, y, z: x*(1-z) + y*(z)
         self.minimum = lambda x, y: npjax.minimum(x, y)
+        self.maximum = lambda x, y: npjax.maximum(x, y)
         self.klimit = npjax.array([18] * embed)
         # module def
         self.module = object
@@ -71,4 +73,4 @@ class RWKVJaxOps(RWKVOp.module):
             return w*(xee2/x2) + b
 
         self.layernorm = ln
-        self.emptyState = npjax.array([[0.01]*embed]*4*layers)
+        self.emptyState = npjax.array([[0.01]*embed]*(4+self.useLogFix)*layers)
