@@ -4,10 +4,10 @@ import rwkvstic.agnostic.backends.base as RWKVOp
 
 class RWKVPTOps(RWKVOp.module):
 
-    def __init__(self, layers, embed, dtype=None):
+    def __init__(self, layers, embed, *args, dtype=None, **kwargs):
         import torch
         import inquirer
-        super().__init__(layers, embed)
+        super().__init__(layers, embed,  dtype=dtype, *args, **kwargs)
         q = [inquirer.List(
             'type',
             message="Load model with which dtype?",
@@ -28,6 +28,7 @@ class RWKVPTOps(RWKVOp.module):
         self.initCpuTensor = lambda x: self.initTensor(x).cpu()
         self.klimit = torch.tensor(
             [18] * embed).to(dtype=self.dtype)
+        self.maximum = torch.maximum
         self.minimum = torch.minimum
         self.sqrt = torch.sqrt
         self.mean = torch.mean
@@ -56,7 +57,7 @@ class RWKVPTOps(RWKVOp.module):
             return torch.layer_norm(x, w.shape, w, b)
         self.layernorm = layernorm
         self.emptyState = torch.zeros(
-            4*layers, embed, dtype=self.dtype)+0.0
+            (4+self.useLogFix)*layers, embed, dtype=self.dtype)+0.0
 
         self.TensorType = torch.Tensor
         self.MatrixType = torch.Tensor
@@ -64,9 +65,9 @@ class RWKVPTOps(RWKVOp.module):
 
 
 class RWKVPTCompatOps(RWKVPTOps):
-    def __init__(self, layers, embed, *args):
+    def __init__(self, layers, embed, *args, **kwargs):
         import torch
-        RWKVPTOps.__init__(self, layers, embed, *args)
+        RWKVPTOps.__init__(self, layers, embed, *args, **kwargs)
         self.relu = lambda x: torch.max(x, torch.zeros_like(x))
         self.matvec = lambda x, y: torch.sum(x*y, dim=1)
 
@@ -108,24 +109,15 @@ class RWKVCudaOps(RWKVPTOps):
         self.klimit = self.klimit.to(dtype=runtimedtype, device='cuda')
 
         self.matvec = lambda x, y: x.mv(
-            y.to(self.dtype)).to(runtimedtype)
-
-        def ln(x, w, b):
-            xee2 = x - self.mean(x)
-
-            x2 = self.sqrt(self.mean(xee2*xee2) + 0.000009999999747378752)
-
-            return w*(xee2/x2) + b
-
-        self.layernorm = ln
+            y.to(dtype=self.dtype)).to(dtype=runtimedtype)
 
         self.emptyState = torch.zeros(
-            4*layers, embed, dtype=runtimedtype, device="cuda")+0.01
+            (4+self.useLogFix)*layers, embed, dtype=runtimedtype, device="cuda")+0.01
 
 
 class RWKVPTTSExportOps(RWKVCudaOps):
-    def __init__(self, layers, embed, *args, includeSampler=None):
-        super().__init__(layers, embed, *args)
+    def __init__(self, layers, embed, *args, includeSampler=None, **kwargs):
+        super().__init__(layers, embed, *args, **kwargs)
         import torch
         import inquirer
         self.stack = lambda x: torch.stack(x)
@@ -147,8 +139,8 @@ class RWKVPTTSExportOps(RWKVCudaOps):
 
 
 class RWKVCudaDeepspeedOps(RWKVCudaOps):
-    def __init__(self, layers, embed, *args):
-        super().__init__(layers, embed, *args)
+    def __init__(self, layers, embed, *args, **kwargs):
+        super().__init__(layers, embed, *args, **kwargs)
 
         try:
             import deepspeed
@@ -160,34 +152,51 @@ class RWKVCudaDeepspeedOps(RWKVCudaOps):
 
 
 class RWKVCudaQuantOps(RWKVPTOps):
-    def __init__(self, layers, embed, *args, runtimedtype=None, useGPU=None, chunksize=None, preQuantized=False):
+    import torch
+
+    def __init__(self, layers, embed, *args, runtimedtype=None, dtype=torch.bfloat16, useGPU=None, chunksize=None, preQuantized=False, maxQuantTarget=None, target=None, **kwargs):
         import torch
         import inquirer
-        super().__init__(layers, embed, torch.bfloat16)
+        super().__init__(layers, embed, *args, dtype=dtype, **kwargs)
 
-        def QuantizeMatrix(x, runtimeDtype, device):
+        def QuantizeMatrix(x, runtimeDtype, device, stream):
             rang = 255
             ran, mini = (x.max(0)[0]-x.min(0)[0])/rang,  x.min(0)[0]
             x = x.double()
             x = ((x-mini)/ran)
+            if stream:
+                x = x.to(
+                    dtype=torch.uint8, non_blocking=True).pin_memory()
+            else:
+                x = x.to(
+                    dtype=torch.uint8, non_blocking=True, device=device)
 
-            x = x.to(
-                dtype=torch.uint8, non_blocking=True, device=device)
-
-            return x, ran.to(runtimeDtype).to(device=device), mini.to(runtimeDtype).to(device=device)
+            return [x, ran.to(runtimeDtype).to(device=device), mini.to(runtimeDtype).to(device=device)]
 
         def QuantizedMatVec(x, y, runtimedtype):
+            if len(x) != 3:
+                return x.mv(y.to(dtype)).to(dtype=runtimedtype)
             rx, spread, zpoint = x
+            y = y.reshape(rx.shape[0], -1)
             yy = y*spread
 
-            rx = rx.to(dtype=runtimedtype)
+            rrx = rx.to(dtype=dtype, device=y.device, non_blocking=True)
+            yy = yy.to(dtype=dtype)
 
-            xmain = rx.matmul(yy.reshape(yy.shape[0], -1, 1)).sum(0).squeeze()
+            xmain = rrx.matmul(yy.reshape(
+                yy.shape[0], -1, 1)).sum(0).squeeze().to(dtype=runtimedtype)
 
             return xmain + torch.tensordot(zpoint, y)
         dev = 'cuda' if (inquirer.confirm(
             "Use GPU?", default=True) if useGPU is None else useGPU) else 'cpu'
 
+        # target is gb before cpu offload
+        target = float(inquirer.text("Target size (in GB):",
+                                     default="100")) if target is None and dev == "cuda" else target
+
+        maxQuantTarget = float(inquirer.text("Max quantization target size (in GB):",
+                                             default="100")) if maxQuantTarget is None and dev == "cuda" else maxQuantTarget
+
         runtimedtype = inquirer.prompt([inquirer.List(
             'type',
             message="Dtype for operations:",
@@ -200,36 +209,47 @@ class RWKVCudaQuantOps(RWKVPTOps):
 
         def initTensor(x):
 
-            if preQuantized and isinstance(x, tuple):
+            if preQuantized and len(x) == 3:
                 return x[0].to(device=dev), x[1].to(dtype=runtimedtype, device=dev), x[2].to(dtype=runtimedtype, device=dev)
 
             if (len(x.shape) != 2):
                 return x.to(dtype=runtimedtype, device=dev)
 
+            if preQuantized:
+                return x.to(dtype=dtype, device=dev)
+
+            dontQuantize = (torch.cuda.max_memory_reserved(
+                0)/1024/1024/1024 > maxQuantTarget) if dev == "cuda" else False
+
+            if dontQuantize:
+                return x.to(dtype=dtype, device=dev)
+
             splitmatrices = torch.chunk(x, chunksize, 1)
-            xx = [QuantizeMatrix(x, runtimedtype, dev)
+            dostream = (torch.cuda.max_memory_reserved(
+                0)/1024/1024/1024 > target) if dev == "cuda" else False
+            xx = [QuantizeMatrix(x, runtimedtype, dev, dostream)
                   for x in splitmatrices]
             xxo = torch.stack([x[0] for x in xx])
             xx1 = torch.stack([x[1] for x in xx])
             xx2 = torch.stack([x[2] for x in xx])
             return xxo, xx1, xx2
-
         self.initTensor = initTensor
+        self.stack = lambda x: torch.stack(
+            x) if isinstance(x[0], torch.Tensor) else x
         self.initCpuTensor = lambda x: x.to(dtype=runtimedtype)
         self.processEmbed = lambda x: x.to(device=dev)
 
         self.postProcessModule = lambda x: x
 
         def matvec(x, y):
-            splitVectors = y.reshape(chunksize, -1)
-            return QuantizedMatVec(x, splitVectors, runtimedtype)
+            return QuantizedMatVec(x, y, runtimedtype)
 
         self.matvec = matvec
 
         self.klimit = self.klimit.to(dtype=runtimedtype, device=dev)
 
         self.emptyState = torch.zeros(
-            4*layers, embed, dtype=runtimedtype, device=dev)+0.01
+            (4+self.useLogFix)*layers, embed, dtype=runtimedtype, device=dev)+0.01
 
 
 # class RWKVPoptorchOps(RWKVPTOps):
@@ -245,10 +265,10 @@ class RWKVCudaQuantOps(RWKVPTOps):
 class RWKVStreamBigOps(RWKVPTOps):
     import torch
 
-    def __init__(self, layers, embed, runtimedtype=torch.float32, dtype=torch.bfloat16, target=None, pinMem=None):
+    def __init__(self, layers, embed, *args, runtimedtype=torch.float32, dtype=torch.bfloat16, target=None, pinMem=None, **kwargs):
         import inquirer
         import torch
-        super().__init__(layers, embed, dtype=dtype)
+        super().__init__(layers, embed, *args, **kwargs)
 
         pinMem = inquirer.prompt([inquirer.Confirm(
             'type',
@@ -270,7 +290,7 @@ class RWKVStreamBigOps(RWKVPTOps):
         self.matvec = lambda z, y: z.cuda(non_blocking=True).mv(
             y.to(dtype)).to(runtimedtype)
         self.emptyState = torch.zeros(
-            4*layers, embed, dtype=runtimedtype, device="cuda")+0.01
+            (4+self.useLogFix)*layers, embed, dtype=runtimedtype, device="cuda")+0.01
 
         def ln(x, w, b):
             xee2 = x - self.mean(x)
@@ -285,8 +305,8 @@ class RWKVStreamBigOps(RWKVPTOps):
 class RWKVSplitCudaOps(RWKVPTOps):
     import torch
 
-    def __init__(self, layers, embed, runtimedtype=torch.float32, dtype=torch.bfloat16, target=None):
-        super().__init__(layers, embed, dtype=dtype)
+    def __init__(self, layers, embed, *args, runtimedtype=torch.float32, dtype=torch.bfloat16, target=None, **kwargs):
+        super().__init__(layers, embed, *args, dtype=dtype, **kwargs)
         import inquirer
         import torch
 
@@ -300,9 +320,9 @@ class RWKVSplitCudaOps(RWKVPTOps):
         # for everything in self, if its a tensor, send to cuda
         # self.matvec = lambda x, y: x.mv(y.to(torch.float16)).to(runtimedtype)
         self.emptyState = torch.zeros(
-            4*layers, embed, dtype=runtimedtype, device="cuda")+0.01
+            (4+self.useLogFix)*layers, embed, dtype=runtimedtype, device="cuda")+0.01
 
-        self.minimum = lambda x, y: torch.min(x, torch.ones_like(x)*30)
+        self.minimum = torch.minimum
 
         def matvec(matx, y):
             chunks = list(map(lambda xx: xx[1].to(
