@@ -20,20 +20,26 @@ def loadContext(model, ctx, newctx, statex, progressCallBack=lambda x: x):
 
 
 class RWKVMaster():
-    def __init__(self, model, emptyState, initTensor=lambda x: x, sampler=None):
+    def __init__(self, model, emptyState, initTensor=lambda x: x, sampler=None, tokPath=None):
         self.model = model
-        self.tokenizer = tokenizer.tokenizer
+
+        self.tokenizer = tokenizer.tokenizer(tokPath)
+
         self.emptyState = emptyState
         self.myState = emptyState
         self.lastToken = 187
         self.initTensor = initTensor
         self.sampler = sampler
 
-    def forward(self, state=None, temp: float = 1.0, top_p_usual: float = 0.8, number=1, stopStrings: List[str] = ["<|endoftext|>"], stopTokens: List[int] = [0], progressLambda=lambda args: args):
+    def forward(self, state=None, temp: float = 1.0, top_p_usual: float = 0.8, number=1, stopStrings: List[str] = ["<|endoftext|>"], stopTokens: List[int] = [0], progressLambda=lambda args: args, end_adj=0.0):
         state = self.myState if state is None else state
         tolens = []
         for i in range(number):
             logits, state = self.model.forward([self.lastToken], state)
+            try:
+                logits[0] += (end_adj)
+            except:
+                pass
             self.myState = state
             sampled = self.sample(
                 logits, temp, top_p_usual) if self.sampler is not None else logits
@@ -45,7 +51,7 @@ class RWKVMaster():
             tolens += [self.lastToken]
             sampled = self.tokenizer.decode(tolens)
             progressLambda(
-                {"logits": logits, "state": state, "output": sampled, "progress": i, "tokens": tolens, "total": number, "current": tokenizer.tokenizer.decode([tolens[-1]])})
+                {"logits": logits, "state": state, "output": sampled, "progress": i, "tokens": tolens, "total": number, "current": self.tokenizer.decode([tolens[-1]])})
             if tolens[-1] in stopTokens:
                 break
             if sampled.endswith((*stopStrings,)):
